/******************************************************************************
 * File: SpacesEditorModule.h
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#pragma once

#include "CoreMinimal.h"
#include "Modules/ModuleInterface.h"
#include "IDetailCustomization.h"
#include "Widgets/SBoxPanel.h"

#define VPADDING_OPTION_SETTING 5
#define HPADDING_OPTION_SETTING 0

DECLARE_LOG_CATEGORY_EXTERN(LogSpacesEditor, Log, All)

struct FSpacesSettings
{
	using FEnableSetting = FReply(*)();
	
	const FName Name;
	const FString Description;
	const FString ButtonEnable;
	const FString ButtonEnabled;
	const FString WarningLog;
	const bool bAskRestart;
	const FEnableSetting EnableSetting;
	bool(* const IsValid)();
	
};

class FSpacesEditorModule : public IModuleInterface
{
public:
	/** IModuleInterface implementation */
	virtual void StartupModule() override;
	virtual void ShutdownModule() override;
	void SetUpAndroidConfig(bool bExportHeadless) const;
	
};

class IDetailLayoutBuilder;
class FSpacesSettingsDetailsCustomization : public IDetailCustomization
{
public:
	/** Makes a new instance of this detail layout class for a specific detail view requesting it */
	static TSharedRef<IDetailCustomization> MakeInstance();

	// IDetailCustomization interface
	virtual void CustomizeDetails(IDetailLayoutBuilder& DetailLayout) override;
	// End of IDetailCustomization interface

	void PreBuildLogs();

private:
	IDetailLayoutBuilder* MyDetailLayout = nullptr;
	
	TSharedRef<SHorizontalBox> CreateSettingButton(const FSpacesSettings setting);
	FReply EnableSetting(const FSpacesSettings setting);
	
	void CheckExtensionCategory(IDetailLayoutBuilder& DetailLayout);
	void CheckPerformanceCategory(IDetailLayoutBuilder& DetailLayout);
	void AskRestart(const bool required);

	static const TArray<FSpacesSettings> MandatorySettings;
	static const TArray<FSpacesSettings> RecommendedPerformanceSettings;
	

};
