/******************************************************************************
 * File: SpacesARCameraImageTexture.cpp
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */


#include "SpacesARCameraImageTexture.h"
#include "Engine/Texture2D.h"
#include "SpacesCommon.h"
#include "Runtime/Launch/Resources/Version.h"

USpacesARCameraImageTexture::USpacesARCameraImageTexture()
{
	
}

USpacesARCameraImageTexture::~USpacesARCameraImageTexture()
{
	
}

void USpacesARCameraImageTexture::UpdateFrameRGBTexture(uint8_t* FrameData, const FVector2D& ImageSize)
{
	if(FrameRGBTexture == nullptr)
	{
		if(!CreateFrameRGBTexture(ImageSize, FName("RGBFrameTexture")))
		{
			UE_LOG(LogSpacesRuntime, Error, TEXT("Spaces AR Camera Image texture: Failed to create RGB texture."));
			return;
		}
	}

	#if ENGINE_MAJOR_VERSION == 5
	FTexture2DMipMap* Mip0 = &FrameRGBTexture->GetPlatformData()->Mips[0];
    #else
	FTexture2DMipMap* Mip0 = &FrameRGBTexture->PlatformData->Mips[0];
	#endif
	FByteBulkData* Mip0ImageData = &Mip0->BulkData;
	
	Mip0ImageData->Lock(LOCK_READ_WRITE);
	uint8* FinalData = (uint8*) Mip0ImageData->Realloc(ImageSize.X * ImageSize.Y * 4);
	FMemory::Memcpy(FinalData, FrameData, sizeof(uint8) * ImageSize.X * ImageSize.Y * 4);
	Mip0ImageData->Unlock();
	
	FrameRGBTexture->UpdateResource();
}

bool USpacesARCameraImageTexture::CreateFrameRGBTexture(const FVector2D& ImageSize, const FName& ImageName)
{
	FrameRGBTexture = UTexture2D::CreateTransient(ImageSize.X, ImageSize.Y, EPixelFormat::PF_B8G8R8A8, ImageName);
	if(FrameRGBTexture == nullptr) return false;
	UE_LOG(LogSpacesRuntime, Error, TEXT("Spaces AR Camera Image texture: RGB texture created."));
	return true;
}


