/******************************************************************************
 * File: SpacesCameraAccessExtension.h
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#pragma once

#include "CoreMinimal.h"
#include "SpacesExtension.h"

class USpacesARCameraImageTexture;

class FSpacesCameraAccessExtension : public FSpacesCaptureExtension
{
public:

	FSpacesCameraAccessExtension();
	virtual ~FSpacesCameraAccessExtension() override;
	
	virtual UARTexture* OnGetARTexture(EARTextureType TextureType) const override final;
	virtual bool OnGetCameraIntrinsics(FARCameraIntrinsics& OutCameraIntrinsics) const override final;
	
	/** IOpenXRExtensionPlugin implementation */
	virtual FString GetDisplayName() override;
	virtual void PostCreateInstance(XrInstance InInstance) override;
	virtual void PostCreateSession(XrSession InSession) override;
	virtual bool GetRequiredExtensions(TArray<const ANSICHAR*>& OutExtensions) override;
	virtual const void* OnCreateSession(XrInstance InInstance, XrSystemId InSystem, const void* InNext) override;
	virtual const void* OnBeginSession(XrSession InSession, const void* InNext) override;
	virtual void UpdateDeviceLocations(XrSession InSession, XrTime DisplayTime, XrSpace TrackingSpace) override;

	void SetCameraAccessState(bool Active);

	bool GetCameraFrameConfigFound() const;

protected:
#if PLATFORM_ANDROID
	virtual void GetRequiredPermissions(TArray<FString>& Permissions) const override;
#endif
	
	
	/** SpacesCaptureExtension implementation */
public:
	virtual bool GetSupportedCaptureTypes(TArray<EARCaptureType>& OutTypes) override;
protected:
	virtual bool OnEnabledCaptureType(EARCaptureType InType) override;
	virtual bool OnDisabledCaptureType(EARCaptureType InType) override;
	virtual bool CanEnableExtension() const override;
	virtual bool OnStart() override;
	virtual void OnPause() override;
	virtual void OnResume() override;
	virtual bool OnStop() override;
	
private:

	class FSpacesSharedARTextureHolder : public FGCObject
	{
	public:
		virtual void AddReferencedObjects(FReferenceCollector& Collector) override;

		USpacesARCameraImageTexture* ARCameraRGBTexture = nullptr;
	};
	
	TMap<EARTextureType, XrCameraInfoQCOMX> CamerasInfosMap;
	TMap<EARTextureType, XrCameraFrameConfigurationQCOMX> CamerasConfigInfosMap;
	TMap<EARTextureType, XrCameraQCOMX> XrCameraHandles;
	int SelectedCameraSensorId = 0;

	TSharedPtr<FSpacesSharedARTextureHolder> ARTextureHolder;
	FARCameraIntrinsics CurrentCameraIntrinsics;

	bool UpdateFrames = false;

	bool bCameraFrameConfigFound = false;
	
	bool GetCamerasInfo(XrSession InSession);
	XrCameraFrameConfigurationQCOMX GetCamerasConfigurationInfo(XrSession InSession, const XrCameraInfoQCOMX& CurrentCameraInfo);
	void CreateCameraHandles(XrSession InSession);
	
	void ShutdownCamera();

	void ExtractRGBFrame(EARTextureType TextureType, TArray<XrCameraFrameBufferQCOMX>& FrameBuffers, const XrCameraSensorPropertiesQCOMX& Sensor, uint8_t*& FramePixels) const;

	void UpdateCameraFrame(EARTextureType TextureType);
	void UpdateCameraIntrinsics(const XrCameraSensorPropertiesQCOMX& Sensor);
};
