﻿/******************************************************************************
 * File: FSpacesCapture.cpp
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#include "SpacesCapture.h"
#include "SpacesRuntimeModule.h"

bool FSpacesCapture::IsEnabled() const
{
	for(const auto SpacesCapture : Captures)
	{
		if(SpacesCapture->IsEnabled()){ return true;}
	}
	return false;		
}

bool FSpacesCapture::TryAddCapture(ESpacesExtension Extension)
{
    //TODO(AS): Add checks for this downcast
	FSpacesCaptureExtension* Capture = static_cast<FSpacesCaptureExtension*>(FSpacesRuntimeModule::Get().GetLoadedExtension(Extension));
	if(Capture == nullptr)
	{
		UE_LOG(LogSpacesRuntime, Warning, TEXT("Trying to add extension %s, but is not lodaded."), SnapdragonSpaces::SpacesExtensionToStr(Extension));
		return false;
	}
	Captures.Add(Capture);
	return true;
}

bool FSpacesCapture::OnToggleARCapture(const bool bOnOff)
{
	bool bSuccess = false;
	for(const auto SpacesCapture : Captures)
	{
		if(bOnOff == true)
		{
			bSuccess = SpacesCapture->EnableCaptureType(GetType()) || bSuccess;
		}
		else
		{
			bSuccess = SpacesCapture->DisableCaptureType(GetType()) || bSuccess;
		}
	}
	return bSuccess;
}

TArray<FARTraceResult> FSpacesCapture::OnLineTraceTrackedObjects(const TSharedPtr<FARSupportInterface, ESPMode::ThreadSafe> ARCompositionComponent, const FVector Start, const FVector End, const EARLineTraceChannels TraceChannels)
{
	TArray<FARTraceResult> Results;

	for (const auto CustomCapture : Captures)
	{
		Results += CustomCapture->OnLineTraceTrackedObjects(ARCompositionComponent, Start, End, TraceChannels);
	}
	return Results;
}
