﻿/******************************************************************************
 * File: SpacesHitTestingExtension.h
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#pragma once

#include "CoreMinimal.h"
#include "SpacesExtension.h"
#include "IOpenXRExtensionPlugin.h"

class FSpacesHitTestingExtension :  public FSpacesCaptureExtension
{
	
public:
	FSpacesHitTestingExtension();
	virtual ~FSpacesHitTestingExtension() override;

	/** IOpenXRExtensionPlugin implementation */
	virtual FString GetDisplayName() override;
	
	/** IOpenXRExtensionPlugin implementation */
	virtual void PostCreateInstance(XrInstance InInstance) override;
	virtual void PostCreateSession(XrSession InSession) override;
	virtual const void* OnCreateSession(XrInstance InInstance, XrSystemId InSystem, const void* InNext) override;

	/** IOpenXRCustomCaptureSupport implementation */
	virtual bool GetRequiredExtensions(TArray<const ANSICHAR*>& OutExtensions) override;
	virtual TArray<FARTraceResult> OnLineTraceTrackedObjects(const TSharedPtr<FARSupportInterface, ESPMode::ThreadSafe> ARCompositionComponent, const FVector Start, const FVector End, const EARLineTraceChannels TraceChannels) override;

	/** SpacesExtension implementation */
protected:
#if PLATFORM_ANDROID
	virtual void GetRequiredPermissions(TArray<FString>& Permissions) const override;
	virtual bool IsRuntimeCameraPermissionsRequired() const override {return true;};
#endif

	/** SpacesCaptureExtension implementation */
	public:
	virtual bool GetSupportedCaptureTypes(TArray<EARCaptureType>& OutTypes) override;
protected:
	virtual bool OnEnabledCaptureType(EARCaptureType InType) override;
	virtual bool OnDisabledCaptureType(EARCaptureType InType) override;
	virtual bool CanEnableExtension() const override;
	virtual bool OnStart() override;
	virtual void OnPause() override;
	virtual void OnResume() override;
	virtual bool OnStop() override;
	
private:
	
	IXRTrackingSystem* XRTrackingSystem = nullptr;
	TArray<XrRayCastTargetTypeQCOM> SupportedTargetTypes;

	SnapdragonSpaces::FRayCastQCOMHandle Ray; //Ray cast handle

};
