/******************************************************************************
 * File: SpacesInputHostControllerExtension.cpp
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#include "SpacesInputHostControllerExtension.h"

#include "SpacesCommon.h"
#include "OpenXRCore.h"
#include "InputCoreTypes.h"

#define INPUT_DEVICE "HostController"

#include "SpacesInputHelper.h"

#define LOCTEXT_NAMESPACE "FSpacesInputHostControllerExtension"


FSpacesInputHostControllerExtension::FSpacesInputHostControllerExtension()
{
	UE_LOG(LogSpacesRuntime, Log, TEXT("Startup SpacesInputHostControllerModule"));
	RegisterOpenXRExtensionModularFeature();
	
	ADD_KEY(Left, Menu, Click, CLICK_FLAGS, "(L) Menu");
	ADD_KEY(Left, Squeeze, Click, CLICK_FLAGS, "(L) Squeeze");
	ADD_KEY(Left, Trigger, Value, VALUE_FLAGS, "(L) Trigger");
	ADD_KEY(Left, Thumbstick, X, VALUE_FLAGS, "(L) Thumbstick X");
	ADD_KEY(Left, Thumbstick, Y, VALUE_FLAGS, "(L) Thumbstick Y");
	ADD_KEY(Left, Thumbstick, Click, CLICK_FLAGS, "(L) Thumbstick Click");
	ADD_KEY(Left, Trackpad, X, VALUE_FLAGS, "(L) Trackpad X");
	ADD_KEY(Left, Trackpad, Y, VALUE_FLAGS, "(L) Trackpad Y");
	ADD_KEY(Left, Trackpad, Click, CLICK_FLAGS, "(L) Trackpad Click");
	ADD_KEY(Left, Trackpad, Touch, CLICK_FLAGS, "(L) Trackpad Touch");
	 
	ADD_KEY(Right, Menu, Click, CLICK_FLAGS, "(R) Menu");
	ADD_KEY(Right, Squeeze, Click, CLICK_FLAGS, "(R) Squeeze");
	ADD_KEY(Right, Trigger, Value, VALUE_FLAGS, "(R) Trigger");
	ADD_KEY(Right, Thumbstick, X, VALUE_FLAGS, "(R) Thumbstick X");
	ADD_KEY(Right, Thumbstick, Y, VALUE_FLAGS, "(R) Thumbstick Y");
	ADD_KEY(Right, Thumbstick, Click, CLICK_FLAGS, "(R) Thumbstick Click");
	ADD_KEY(Right, Trackpad, X, VALUE_FLAGS, "(R) Trackpad X");
	ADD_KEY(Right, Trackpad, Y, VALUE_FLAGS, "(R) Trackpad Y");
	ADD_KEY(Right, Trackpad, Click, CLICK_FLAGS, "(R) Trackpad Click");
	ADD_KEY(Right, Trackpad, Touch, CLICK_FLAGS, "(R) Trackpad Touch");
}

FSpacesInputHostControllerExtension::~FSpacesInputHostControllerExtension()
{
	UE_LOG(LogSpacesRuntime, Log, TEXT("Shutdown SpacesInputHostControllerModule"));
	UnregisterOpenXRExtensionModularFeature();
}

bool FSpacesInputHostControllerExtension::GetInteractionProfile(XrInstance InInstance, FString& OutKeyPrefix, XrPath& OutPath,
                                               bool& bOutHasHaptics)
{
    UE_LOG(LogSpacesRuntime, Verbose, TEXT("Spaces GetInteractionProfile"));
    OutKeyPrefix = INPUT_CATEGORY;
	if(!XR_SUCCEEDED(OpenXRDynamicAPI::xrStringToPath(InInstance, "/interaction_profiles/microsoft/motion_controller", &InteractionProfile))) return false;
	OutPath = InteractionProfile;
    bOutHasHaptics = true;

    return true;
}

bool FSpacesInputHostControllerExtension::GetControllerModel(XrInstance InInstance, XrPath InInteractionProfile,
	XrPath InDevicePath, FSoftObjectPath& OutPath)
{

	if(InInteractionProfile != InteractionProfile) return false;
	
	OutPath = "/SnapdragonSpaces/Devices/HostController/SM_HostController.SM_HostController";
	return true;
}

FString FSpacesInputHostControllerExtension::GetDisplayName()
{
	return FString(TEXT("Spaces Input Host Controller"));
}

void FSpacesInputHostControllerExtension::SetHapticsDuration(float Duration)
{
	HapticsDuration = Duration;
}

float FSpacesInputHostControllerExtension::GetHapticsDuration() const
{
	return HapticsDuration;
}

void FSpacesInputHostControllerExtension::SetHapticsFrequency(float Frequency)
{
	HapticsFrequency = Frequency;
}

float FSpacesInputHostControllerExtension::GetHapticsFrequency() const
{
	return HapticsFrequency;
}

void FSpacesInputHostControllerExtension::SetHapticsAmplitude(float Amplitude)
{
	HapticsAmplitude = Amplitude;
}

float FSpacesInputHostControllerExtension::GetHapticsAmplitude() const
{
	return HapticsAmplitude;
}

#undef LOCTEXT_NAMESPACE
