/******************************************************************************
 * File: SpacesInputXRControllerExtension.cpp
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#include "SpacesInputXRControllerExtension.h"

#include "SpacesCommon.h"
#include "OpenXRCore.h"
#include "InputCoreTypes.h"

#define INPUT_DEVICE "XRController"

#include "SpacesInputHelper.h"

#define LOCTEXT_NAMESPACE "FSpacesInputXRControllerExtension"


FSpacesInputXRControllerExtension::FSpacesInputXRControllerExtension()
{
	UE_LOG(LogSpacesRuntime, Log, TEXT("Startup SpacesInputXRControllerModule"));
	RegisterOpenXRExtensionModularFeature();

	ADD_KEY(Left, X, Click, CLICK_FLAGS, "(L) X");
	ADD_KEY(Left, X, Touch, CLICK_FLAGS, "(L) X Touch");
	ADD_KEY(Left, Y, Click, CLICK_FLAGS, "(L) Y");
	ADD_KEY(Left, Y, Touch, CLICK_FLAGS, "(L) Y Touch");
	ADD_KEY(Left, Menu, Click, CLICK_FLAGS, "(L) Menu");
	ADD_KEY(Left, Squeeze, Value, VALUE_FLAGS, "(L) Squeeze");
	ADD_KEY(Left, Trigger, Value, VALUE_FLAGS, "(L) Trigger");
	ADD_KEY(Left, Trigger, Touch, CLICK_FLAGS, "(L) Trigger Touch");
	ADD_KEY(Left, Thumbstick, X, VALUE_FLAGS, "(L) Thumbstick X");
	ADD_KEY(Left, Thumbstick, Y, VALUE_FLAGS, "(L) Thumbstick Y");
	ADD_KEY(Left, Thumbstick, Click, CLICK_FLAGS, "(L) Thumbstick Click");
	ADD_KEY(Left, Thumbstick, Touch, CLICK_FLAGS, "(L) Thumbstick Touch");
	ADD_KEY(Left, Thumbrest, Touch, CLICK_FLAGS, "(L) Thumbrest Touch");

	ADD_KEY(Right, A, Click, CLICK_FLAGS, "(R) A");
	ADD_KEY(Right, A, Touch, CLICK_FLAGS, "(R) A Touch");
	ADD_KEY(Right, B, Click, CLICK_FLAGS, "(R) B");
	ADD_KEY(Right, B, Touch, CLICK_FLAGS, "(R) B Touch");
	ADD_KEY(Right, System, Click, CLICK_FLAGS, "(R) System");
	ADD_KEY(Right, Squeeze, Value, VALUE_FLAGS, "(R) Squeeze");
	ADD_KEY(Right, Trigger, Value, VALUE_FLAGS, "(R) Trigger");
	ADD_KEY(Right, Trigger, Touch, CLICK_FLAGS, "(R) Trigger Touch");
	ADD_KEY(Right, Thumbstick, X, VALUE_FLAGS, "(R) Thumbstick X");
	ADD_KEY(Right, Thumbstick, Y, VALUE_FLAGS, "(R) Thumbstick Y");
	ADD_KEY(Right, Thumbstick, Click, CLICK_FLAGS, "(R) Thumbstick Click");
	ADD_KEY(Right, Thumbstick, Touch, CLICK_FLAGS, "(R) Thumbstick Touch");
	ADD_KEY(Right, Thumbrest, Touch, CLICK_FLAGS, "(R) Thumbrest Touch");
}

FSpacesInputXRControllerExtension::~FSpacesInputXRControllerExtension()
{
	UE_LOG(LogSpacesRuntime, Log, TEXT("Shutdown SpacesInputXRControllerModule"));
	UnregisterOpenXRExtensionModularFeature();
}

bool FSpacesInputXRControllerExtension::GetInteractionProfile(XrInstance InInstance, FString& OutKeyPrefix, XrPath& OutPath,
                                               bool& bOutHasHaptics)
{
    UE_LOG(LogSpacesRuntime, Verbose, TEXT("Spaces GetInteractionProfile"));
    OutKeyPrefix = INPUT_CATEGORY;
    if(!XR_SUCCEEDED(OpenXRDynamicAPI::xrStringToPath(InInstance, "/interaction_profiles/oculus/touch_controller", &InteractionProfile))) return false;
	OutPath = InteractionProfile;
    bOutHasHaptics = true;

    return true;
}

bool FSpacesInputXRControllerExtension::GetControllerModel(XrInstance InInstance, XrPath InInteractionProfile,
	XrPath InDevicePath, FSoftObjectPath& OutPath) 
{
	
	if(InInteractionProfile != InteractionProfile) return false;
	
	OutPath = "/SnapdragonSpaces/Devices/XRController/SM_XRController.SM_XRController";
	return true;
}

FString FSpacesInputXRControllerExtension::GetDisplayName()
{
	return FString(TEXT("Spaces Input XRController"));
}

#undef LOCTEXT_NAMESPACE
