﻿/******************************************************************************
 * File: JavaClassObjectExtended.h
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#pragma once

#include "CoreMinimal.h"

#if PLATFORM_ANDROID
#if USE_ANDROID_JNI
#include "Android/AndroidJava.h"

namespace AndroidJavaEnv
{
    FScopedJavaObject<jobject> GetApplicationContextThis();
}

class FJavaClassObjectExtended : public FJavaClassObject
{

public:
    
    using FJavaClassObject::FJavaClassObject;
    
    
protected:

    jfieldID FindField(const char* FieldName, const char* FieldType);

    template<typename Type>
    Type GetField(jfieldID FieldID);
    
    template<typename Type>
    void SetField(jfieldID FieldID, const Type& Value);
    
    FJavaClassMethod GetStaticClassMethod(const char* MethodName, const char* FuncSig);
    
    template<typename ReturnType>
    ReturnType CallStaticMethod(FJavaClassMethod Method, ...);
    
};

template<>
bool FJavaClassObjectExtended::GetField<bool>(jfieldID FieldID);

template<>
int FJavaClassObjectExtended::GetField<int>(jfieldID FieldID);

template<>
jobject FJavaClassObjectExtended::GetField<jobject>(jfieldID FieldID);

template<>
jobjectArray FJavaClassObjectExtended::GetField<jobjectArray>(jfieldID FieldID);

template<>
int64 FJavaClassObjectExtended::GetField<int64>(jfieldID FieldID);

template<>
FString FJavaClassObjectExtended::GetField<FString>(jfieldID FieldID);



template<>
void FJavaClassObjectExtended::SetField<bool>(jfieldID FieldID, const bool& Value);

template<>
void FJavaClassObjectExtended::SetField<int>(jfieldID FieldID, const int& Value);

template<>
void FJavaClassObjectExtended::SetField<jobject>(jfieldID FieldID, const jobject& Value);

template<>
void FJavaClassObjectExtended::SetField<jobjectArray>(jfieldID FieldID, const jobjectArray& Value);

template<>
void FJavaClassObjectExtended::SetField<int64>(jfieldID FieldID, const int64& Value);

template<>
void FJavaClassObjectExtended::SetField<FString>(jfieldID FieldID, const FString& Value);



template<>
void FJavaClassObjectExtended::CallStaticMethod<void>(FJavaClassMethod Method, ...);

template<>
bool FJavaClassObjectExtended::CallStaticMethod<bool>(FJavaClassMethod Method, ...);

template<>
int FJavaClassObjectExtended::CallStaticMethod<int>(FJavaClassMethod Method, ...);

template<>
jobject FJavaClassObjectExtended::CallStaticMethod<jobject>(FJavaClassMethod Method, ...);

template<>
jobjectArray FJavaClassObjectExtended::CallStaticMethod<jobjectArray>(FJavaClassMethod Method, ...);

template<>
int64 FJavaClassObjectExtended::CallStaticMethod<int64>(FJavaClassMethod Method, ...);

template<>
FString FJavaClassObjectExtended::CallStaticMethod<FString>(FJavaClassMethod Method, ...);

#endif //USE_ANDROID_JNI
#endif //PLATFORM_ANDROID