﻿/******************************************************************************
 * File: PermissionHelperWrapper.cpp
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#include "PermissionHelperWrapper.h"

#if PLATFORM_ANDROID
#include "Android/AndroidApplication.h"

#if USE_ANDROID_JNI
#include "Android/AndroidJNI.h"

FName FPermissionHelperWrapper::GetClassName()
{
	return FName("com/google/vr/sdk/samples/permission/PermissionHelper");
}

FPermissionHelperWrapper::FPermissionHelperWrapper()
	: FJavaClassObjectExtended(GetClassName(), "()V")
	, CheckPermissionMethod(GetStaticClassMethod("checkPermission", "(Ljava/lang/String;)Z"))
	, AcquirePermissionsMethod(GetStaticClassMethod("acquirePermissions", "([Ljava/lang/String;)V"))
{}

bool FPermissionHelperWrapper::CheckPermission(const FString& Permission)
{
	JNIEnv* env = FAndroidApplication::GetJavaEnv();
	const auto jstringPermission = FJavaHelper::ToJavaString(env, Permission);
	return CallStaticMethod<bool>(CheckPermissionMethod, *jstringPermission);
}

void FPermissionHelperWrapper::AcquirePermissions(const TArray<FString>& Permissions)
{
	JNIEnv* env = FAndroidApplication::GetJavaEnv();
	auto jarrayPermissionsArray = NewScopedJavaObject(env, (jobjectArray)env->NewObjectArray(Permissions.Num(), FJavaWrapper::JavaStringClass, NULL));
	for (int i = 0; i < Permissions.Num(); i++)
	{
		const auto jstringPermission = FJavaHelper::ToJavaString(env, Permissions[i]);
		env->SetObjectArrayElement(*jarrayPermissionsArray, i, *jstringPermission);
	}
	CallStaticMethod<void>(AcquirePermissionsMethod, *jarrayPermissionsArray);
}

#endif //USE_ANDROID_JNI
#endif //PLATFORM_ANDROID