/******************************************************************************
 * File: SpacesPlaneDetectionExtension.h
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */ 

#pragma once

#include "CoreMinimal.h"
#include "SpacesExtension.h"

#include "ARSessionConfig.h"

class IOpenXRARTrackedMeshHolder;
class IXRTrackingSystem;

namespace SnapdragonSpaces
{

	FORCEINLINE void XrPlaneExtentToUnrealPlane(const XrPlaneExtentQCOM& PlaneExtent, FVector& CenterOffset, FVector& Extents, float Scale = 1.0f);
	FORCEINLINE FTransform CalculatePlaneTransform(const XrPosef& Pose, const FVector& CenterOffset, float Scale = 1.0f);

	XrPlaneFilterQCOM PlaneDetectionModeToXrPlaneFilter(EARPlaneDetectionMode Mode);
	EARPlaneOrientation XrPlaneTypeToPlaneOrientation(XrPlaneTypeQCOM Type);

}

class FSpacesPlaneDetectionExtension : public FSpacesCaptureExtension
{
public:

	FSpacesPlaneDetectionExtension();
	virtual ~FSpacesPlaneDetectionExtension() override;

	/** IOpenXRExtensionPlugin implementation */
	virtual void PostCreateInstance(XrInstance InInstance) override;
	virtual const void* OnCreateSession(XrInstance InInstance, XrSystemId InSystem, const void* InNext) override;
	virtual void PostCreateSession(XrSession InSession) override;
	virtual FString GetDisplayName() override;
    virtual bool GetRequiredExtensions(TArray<const ANSICHAR*>& OutExtensions) override;
    virtual void UpdateDeviceLocations(XrSession InSession, XrTime DisplayTime, XrSpace TrackingSpace) override;


	/** SpacesExtension implementation */
protected:
#if PLATFORM_ANDROID
	virtual void GetRequiredPermissions(TArray<FString>& Permissions) const override;
#endif
	
	/** SpacesCaptureExtension implementation */
public:
	virtual bool GetSupportedCaptureTypes(TArray<EARCaptureType>& OutTypes) override;
protected:
	virtual bool OnEnabledCaptureType(EARCaptureType InType) override;
	virtual bool OnDisabledCaptureType(EARCaptureType InType) override;
	virtual bool CanEnableExtension() const override;
	virtual bool OnStart() override;
	virtual void OnPause() override;
	virtual void OnResume() override;
	virtual bool OnStop() override;
	
private:
	
	TArray<FGuid> PreviousPlanesId;
	
	IXRTrackingSystem* XRTrackingSystem = nullptr;
	IOpenXRARTrackedMeshHolder* TrackedMeshHolder = nullptr;

	SnapdragonSpaces::FPlaneDetectionQCOMHandle PlaneDetection;
	
	EARPlaneDetectionMode PlaneDetectionMode = EARPlaneDetectionMode::None;
	
	bool bDeviceSupportsPlaneDetection = false;

	bool TryCreatePlaneDetection(EARPlaneDetectionMode Mode, bool EnableConvexHull);

	TArray<XrVector3f> GetPlaneVertices(uint32_t ConvexHullId) const;
	
	bool LocatePlanes(XrSpace Space, XrTime Time, TMap<FGuid, FTrackedPlaneHolder>& LocatedPlanes) const;
	void CalculateMissingPlanes(const TMap<FGuid, FTrackedPlaneHolder>& LocatedPlanes, TArray<FGuid>& MissingPlanesId);
	void RemovePlanes(const TArray<FGuid>& MissingPlanesId) const;
	void AllocatePlanes(TMap<FGuid, FTrackedPlaneHolder>& Planes) const;
	void ClearPlanes() const;

};
