/******************************************************************************
 * File: SpacesRuntimeModule.cpp
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#include "SpacesRuntimeModule.h"

#include "SpacesCommon.h"
#include "SpacesExtension.h"

/****** Include Extensions ******/
#include "SpacesCoreExtension.h"
#include "Input/SpacesInputXRControllerExtension.h"
#include "Input/SpacesInputHostControllerExtension.h"
#include "SpatialAnchor/SpacesSpatialAnchorExtension.h"
#include "PlaneDetection/SpacesPlaneDetectionExtension.h"
#include "ImageTracking/SpacesImageTrackingExtension.h"
#include "Meshing/SpacesMeshingExtension.h"
#include "HandTracking/SpacesHandTrackingExtension.h"
#include "HitTesting/SpacesHitTestingExtension.h"
#include "CameraAccess/SpacesCameraAccessExtension.h"
/********************************/

#include "Modules/ModuleManager.h"

#define LOCTEXT_NAMESPACE "FSpacesRuntimeModule"

#define SPACES_LOAD_REQUIRED_EXTENSION(Name)												\
	if(auto Extension = FSpacesExtension::TryCreateExtension<FSpaces##Name##Extension>())	\
	{																						\
		LoadedExtensions.Add(ESpacesExtension::Name, MoveTemp(Extension));					\
	}																						\
	else																					\
	{																						\
		UE_LOG(LogSpacesRuntime, Error, TEXT("Error loading required extension: "#Name));	\
		LoadedExtensions.Empty();															\
		return;																				\
	}


#define SPACES_LOAD_EXTENSION(Name)																								\
	{																															\
		bLoadExtension = true;																									\
		GConfig->GetBool(TEXT("/Script/SpacesRuntime.SpacesRuntimeSettings"), TEXT("bLoad"#Name), bLoadExtension, GEngineIni);	\
		if(bLoadExtension){																										\
			if(auto Extension = FSpacesExtension::TryCreateExtension<FSpaces##Name##Extension>())								\
			{																													\
				LoadedExtensions.Add(ESpacesExtension::Name, MoveTemp(Extension));												\
			}																													\
		}																														\
	}															

void FSpacesRuntimeModule::LoadSpacesExtensions()
{
	SPACES_REQUIRED_EXTENSION_LIST(SPACES_LOAD_REQUIRED_EXTENSION)
	if(GConfig->IsReadyForUse())
	{
		bool bLoadExtension = true;
		SPACES_EXTENSION_LIST(SPACES_LOAD_EXTENSION);
	}
}

FSpacesRuntimeModule& FSpacesRuntimeModule::Get()
{
    return FModuleManager::Get().LoadModuleChecked<FSpacesRuntimeModule>("SpacesRuntime");
}

const TMap<ESpacesExtension, TUniquePtr<FSpacesExtension>>& FSpacesRuntimeModule::GetLoadedExtensions() const
{
	return LoadedExtensions;
}

FSpacesExtension* FSpacesRuntimeModule::GetLoadedExtension(ESpacesExtension Extension) const
{
	if(!LoadedExtensions.Contains(Extension)) return nullptr;
	return LoadedExtensions[Extension].Get();
}

void FSpacesRuntimeModule::StartupModule()
{
	UE_LOG(LogSpacesRuntime, Log, TEXT("Startup SpacesRuntimeModule"));

    LoadSpacesExtensions();
}

void FSpacesRuntimeModule::ShutdownModule()
{
	UE_LOG(LogSpacesRuntime, Log, TEXT("Shutdown SpacesRuntimeModule"));

	LoadedExtensions.Empty();
	
}

#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FSpacesRuntimeModule, SpacesRuntime)