﻿/******************************************************************************
 * File: SpacesRuntimeSettings.cpp
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#include "SpacesRuntimeSettings.h"
#include "SpacesCommon.h"
#include "SpacesExtension.h"
#include "SpacesRuntimeModule.h"
#include "Modules/ModuleManager.h"
#include "Runtime/Launch/Resources/Version.h"

#define LOCTEXT_NAMESPACE "USpacesOpenXRSettings"

USpacesRuntimeSettings::USpacesRuntimeSettings(const FObjectInitializer& ObjectInitializer)
	:Super(ObjectInitializer)
{}

#if WITH_EDITOR

#define CHECK_EXTENSION_DEPRECATED(Name) bLoad##Name = (LoadExtensions & static_cast<__underlying_type(ESpacesExtensionFlags)>(ESpacesExtensionFlags::Name)) == static_cast<__underlying_type(ESpacesExtensionFlags)>(ESpacesExtensionFlags::Name);

void USpacesRuntimeSettings::PostInitProperties()
{
	Super::PostInitProperties();
	int32 LoadExtensions;
	if(GConfig->GetInt(TEXT("/Script/SpacesRuntime.SpacesRuntimeSettings"), TEXT("LoadExtensions"), LoadExtensions, GEngineIni))
	{
		GConfig->RemoveKey(TEXT("/Script/SpacesRuntime.SpacesRuntimeSettings"), TEXT("LoadExtensions"), GEngineIni);
		GConfig->Flush(false,GGameIni);
		SPACES_EXTENSION_LIST(CHECK_EXTENSION_DEPRECATED)
#if ENGINE_MAJOR_VERSION == 4
		UpdateDefaultConfigFile();
#elif ENGINE_MAJOR_VERSION == 5
		TryUpdateDefaultConfigFile();
#endif
	}
}

void USpacesRuntimeSettings::PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent)
{

	const auto Property = PropertyChangedEvent.Property;

	if (Property)
	{
		if(Property->GetFName() == GET_MEMBER_NAME_CHECKED(USpacesRuntimeSettings, bExportHeadless))
		{
			SetUpAndroidConfig();
		}
	}
	
	Super::PostEditChangeProperty(PropertyChangedEvent);
}


void USpacesRuntimeSettings::SetUpAndroidConfig() const
{
	if(!OnUpdateAndroidConfig.ExecuteIfBound(bExportHeadless))
	{
		UE_LOG(LogSpacesRuntime, Warning, TEXT("OnUpdateAndroidConfig not valid execute"));
	}
}

FName USpacesRuntimeSettings::GetContainerName() const
{
    return FName("Project");
}

FName USpacesRuntimeSettings::GetCategoryName() const
{
    return FName("Plugins");
}

FName USpacesRuntimeSettings::GetSectionName() const
{
    return FName("Snapdragon Spaces");
}

FText USpacesRuntimeSettings::GetSectionText() const
{
    return LOCTEXT("SpacesSettingsName","Snapdragon Spaces");
}

FText USpacesRuntimeSettings::GetSectionDescription() const
{
    return LOCTEXT("SpacesSettingsDescription","Configure Snapdragon Spaces settings.");;
}
#endif

#undef LOCTEXT_NAMESPACE