/******************************************************************************
 * File: SpacesSpatialAnchorExtension.h
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#pragma once

#include "CoreMinimal.h"
#include "SpacesExtension.h"
#include "IOpenXRExtensionPlugin.h"

class FSpacesSpatialAnchor;
class FSpacesSpatialAnchorExtension :  public FSpacesExtension, public IOpenXRCustomAnchorSupport
{

public:
    
    /** IOpenXRCustomAnchorSupport implementation */
    virtual bool OnPinComponent(class UARPin* Pin, XrSession InSession, XrSpace TrackingSpace, XrTime DisplayTime, float worldToMeterScale) override;
    virtual void OnRemovePin(class UARPin* Pin) override;
    virtual void OnUpdatePin(class UARPin* Pin, XrSession InSession, XrSpace TrackingSpace, XrTime DisplayTime, float worldToMeterScale) override;
    virtual bool IsLocalPinSaveSupported() const override;
    virtual bool ArePinsReadyToLoad() override;
    virtual void LoadARPins(XrSession InSession, TFunction<UARPin*(FName)> OnCreatePin) override;
    virtual bool SaveARPin(XrSession InSession, FName InName, UARPin* InPin) override;
    virtual void RemoveSavedARPin(XrSession InSession, FName InName) override;
	virtual void RemoveAllSavedARPins(XrSession InSession) override;


	FSpacesSpatialAnchorExtension();
    virtual ~FSpacesSpatialAnchorExtension() override;

	/** IOpenXRExtensionPlugin implementation */
	virtual const void* OnBeginSession(XrSession InSession, const void* InNext) override;
    virtual void PostCreateInstance(XrInstance InInstance) override;
    virtual void PostCreateSession(XrSession InSession) override;
    virtual FString GetDisplayName() override;
	virtual bool GetRequiredExtensions(TArray<const ANSICHAR*>& OutExtensions) override;
    virtual IOpenXRCustomAnchorSupport* GetCustomAnchorSupport() override;
	virtual void UpdateDeviceLocations(XrSession InSession, XrTime DisplayTime, XrSpace TrackingSpace) override;
    /** ISpacesOpenXRExtension implementation */

	TArray<XrSpatialAnchorPersistenceNameMSFT> GetSavedAnchorNames() const;

protected:
	static TMap<TWeakObjectPtr<UARPin>, TSharedPtr<FSpacesSpatialAnchor, ESPMode::ThreadSafe>> LoadAnchors(XrInstance InInstance, XrSession InSession, XrSpatialAnchorStoreConnectionMSFT AnchorStore, TMap<FName, TWeakObjectPtr<UARPin>> AnchorToLoad);
	static bool SaveAnchor(XrSpatialAnchorStoreConnectionMSFT AnchorStore, FName InName, UARPin* InPin);
	static bool InitializePersistenceName(FName InName, XrSpatialAnchorPersistenceNameMSFT& PersistenceName);
	
private:
	static bool CreateSpatialAnchorFromPersistenceName(XrSession InSession, XrSpatialAnchorStoreConnectionMSFT AnchorStore, XrSpatialAnchorMSFT& SpatialAnchor, XrSpatialAnchorPersistenceNameMSFT PersistenceName);
	bool CreateAsyncSaveThread();
	bool UpdateAsyncLoading();
	bool UpdateAsyncSaving();
	
	XrSpatialAnchorStoreConnectionMSFT AnchorStore = XR_NULL_HANDLE;
	IXRTrackingSystem* XRTrackingSystem = nullptr;
	
	TFuture<TMap<TWeakObjectPtr<UARPin>, TSharedPtr<FSpacesSpatialAnchor, ESPMode::ThreadSafe>>> AnchorLoadingFuture;

	FName AnchorSaving;
	TMap<FName, UARPin*> AnchorPendingToSave;
	TFuture<bool> AnchorSavingFuture;
	
};
