/******************************************************************************
 * File: USpacesHGesturesBlueprintLibrary.h
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */
#pragma once

#include "CoreMinimal.h"
#include "Kismet/BlueprintFunctionLibrary.h"
#include "SpacesHGesturesBlueprintLibrary.generated.h"

UENUM(BlueprintType)
enum class ESpacesHandGestureType : uint8
{
	UNKNOWN = 22 UMETA(DisplayName = "Unknown Gesture"), 
	OPEN_HAND = 0 UMETA(DisplayName = "Open hand Gesture"),
	GRAB = 2 UMETA(DisplayName = "Grab Gesture"),
	PINCH = 7 UMETA(DisplayName = "Pinch Gesture"),
	POINT = 8 UMETA(DisplayName = "Point Gesture (Obsolete)", ToolTip = "The POINT gesture is not used and will be removed in the future."),
	VICTORY = 9 UMETA(DisplayName = "Victory Gesture (Obsolete)", ToolTip = "The VICTORY gesture is not used and will be removed in the future."),
	METAL = 11 UMETA(DisplayName = "Metal Gesture (Obsolete)", ToolTip = "The METAL gesture is not used and will be removed in the future."),
	ERROR = 144
};

class UMaterialInterface;

UENUM(BlueprintType)
enum class ESpacesHandType : uint8
{
	LEFT = 0,
	RIGHT = 1
};

USTRUCT(BlueprintType)
struct FSpacesHandGestureData
{
	GENERATED_BODY()
	UPROPERTY(BlueprintReadOnly, Category = "Snapdragon Spaces")
	ESpacesHandGestureType Type;

	UPROPERTY(BlueprintReadOnly, Category = "Snapdragon Spaces")
	float GestureRatio;

	UPROPERTY(BlueprintReadOnly, Category = "Snapdragon Spaces")
	float FlipRatio;
};


UCLASS()
class SPACESRUNTIME_API USpacesHGesturesBlueprintLibrary : public UBlueprintFunctionLibrary
{
	GENERATED_BODY()

public:
	
	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces")
	static FSpacesHandGestureData GetXRHandGestureData(ESpacesHandType HandType);

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces")
	static bool IsHandTracked(ESpacesHandType HandType);

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces", meta=(DeprecatedFunction, DeprecationMessage="Use Toggle ARCapture."))
	static void SetHandMeshStatus(bool Active);

	UFUNCTION(BlueprintCallable, BlueprintPure, Category="Snapdragon Spaces")
	static bool GetHandMeshStatus();

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces", meta=(DeprecatedFunction, DeprecationMessage="Use ARTrackableNotify component."))
	static void RenderHandMesh(ESpacesHandType HandIndex, UMaterialInterface* Material, AActor*& HandMeshActor, int& OriginalVerticesNum);

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces")
	static void SetHandTrackingState(bool Active);
	
};
