﻿/******************************************************************************
 * File: SpacesRuntimeBlueprintLibrary.h
 * Copyright (c) 2022 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#pragma once

#include "CoreMinimal.h"
#include "SpacesGameUserSettings.h"
#include "SpacesComponentVersion.h"
#include "Engine/Classes/Kismet/BlueprintFunctionLibrary.h"
#include "Misc/EnumRange.h"
#include "SpacesRuntimeBlueprintLibrary.generated.h"

class UARPin;
class UMaterialInterface;
enum class EControllerHand : uint8;

UENUM(BlueprintType)
enum class ESpacesExtensionBP : uint8
{
	SpatialAnchor,
	PlaneDetection,
	ImageTracking,
	HandTracking,
	HitTesting,
	Meshing,
	CameraAccess
};
ENUM_RANGE_BY_FIRST_AND_LAST(ESpacesExtensionBP,ESpacesExtensionBP::SpatialAnchor, ESpacesExtensionBP::CameraAccess);

UCLASS()
class SPACESRUNTIME_API USpacesRuntimeBlueprintLibrary : public UBlueprintFunctionLibrary
{
    GENERATED_BODY()

public:

	UFUNCTION(BlueprintCallable, Category = "SnapdragonSpaces")
	static bool GetSpacesComponentVersions(TArray<FSpacesComponentVersion>& OutComponentVersions);
	
	UFUNCTION(BlueprintPure, Category="Snapdragon Spaces|Spatial Anchors")
	static bool GetAnchorInfoFromARPin(UARPin* Pin, int64& AnchorHandle, int64& AnchorSpaceHandle);
	
	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces|Persistent Spatial Anchors")
	static bool LoadSavedAnchor(FName InName);
	
	UFUNCTION(BlueprintPure, Category="Snapdragon Spaces|Persistent Spatial Anchors")
	static TArray<FName> GetSavedAnchorNames();
	
	UFUNCTION(BlueprintPure, Category="Snapdragon Spaces|Persistent Spatial Anchors")
	static FName GetSavedAnchorNameForARPin(UARPin* Pin);

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces", meta=(DeprecatedFunction, DeprecationMessage="UpdateSpacesOriginPositionAndOrientation is deprecated."))
	static void UpdateSpacesOriginPositionAndOrientation(FVector OriginLocation, FRotator OriginRotation, bool bMaintainUpDirection = true) {}

	UFUNCTION(BlueprintPure, Category="Snapdragon Spaces", meta=(DeprecatedFunction, DeprecationMessage="GetSpacesOriginPositionAndOrientation is deprecated, returns always FTransfom()."))
	static FTransform GetSpacesOriginPositionAndOrientation() { return FTransform(); }
	
	UFUNCTION(BlueprintPure, Category="Snapdragon Spaces|Spaces Settings")
	static ESpacesControllerType GetSpacesController();
	
	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces|Spaces Settings")
	static void SetSpacesController(ESpacesControllerType controller);
	
	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces|Spaces Settings")
	static void ResetPose();

	UFUNCTION(BlueprintCallable,BlueprintPure, Category="Snapdragon Spaces|Spaces Settings")
	static FString GetEngineBuildDate();

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces")
	static void RenderSpatialMesh(AActor*& MeshActor, UMaterialInterface* Material, int& PreviousNumberOfIndices);

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces")
	static void SetSpatialMeshingNormalsCalculationState(bool Active);
	
	UFUNCTION(BlueprintCallable, BlueprintPure, Category="Snapdragon Spaces")
	static bool GetSpatialMeshingNormalsCalculationState();
	
	UFUNCTION(BlueprintCallable,BlueprintPure, Category="Snapdragon Spaces")
	static bool IsExtensionLoaded(ESpacesExtensionBP Extension);

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces|Spaces Settings")
	static void SetSpacesHapticsByValue(APlayerController* PlayerController, float Frequency, float Amplitude, float Duration, EControllerHand Hand);

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces|Spaces Settings")
	static void SetCameraFrameAccessState(bool Active);

	UFUNCTION(BlueprintCallable, Category="Snapdragon Spaces|Spaces Settings")
	static bool IsCameraFrameAccessSupported();
	
};