﻿/******************************************************************************
 * File: SpacesRuntimeSettings.h
 * Copyright (c) 2022-2023 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
 *
 * Confidential and Proprietary - Qualcomm Technologies, Inc.
 *
 */

#pragma once

#include "CoreMinimal.h"
#include "SpacesRuntimeBlueprintLibrary.h"
#include "DeveloperSettings/Public/Engine/DeveloperSettings.h"
#include "Engine/Classes/Engine/EngineTypes.h"

#include "SpacesRuntimeSettings.generated.h"

DECLARE_DELEGATE_OneParam(FOnUpdateAndroidConfig, bool);

UCLASS(config = Engine, defaultconfig)
class SPACESRUNTIME_API USpacesRuntimeSettings : public UDeveloperSettings
{
    GENERATED_BODY()

public:
	
	/************* Extension load settings *************/
	
	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, DisplayName="Spatial Anchor", Category="Extensions", meta = (ConfigRestartRequired=true))
	bool bLoadSpatialAnchor = true;
	
	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, DisplayName="Plane Detection", Category="Extensions", meta = (ConfigRestartRequired=true))
	bool bLoadPlaneDetection = true;
	
	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, DisplayName="Image Tracking", Category="Extensions", meta = (ConfigRestartRequired=true))
	bool bLoadImageTracking = true;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, DisplayName="Hand Tracking", Category="Extensions", meta = (ConfigRestartRequired=true))
	bool bLoadHandTracking = true;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, DisplayName="Spatial Meshing (Experimental)", Category="Extensions", meta = (ConfigRestartRequired=true))
	bool bLoadMeshing = true;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, DisplayName="Hit Testing", Category="Extensions", meta = (ConfigRestartRequired=true))
	bool bLoadHitTesting = true;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, DisplayName="Camera Access (Experimental)", Category="Extensions", meta = (ConfigRestartRequired=true))
	bool bLoadCameraAccess = true;
		
	/**************************************************/
	
	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, Category="General")
	bool bCheckInstalledRuntime = true;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, Category="General")
	bool bLaunchAppOnViewer = true;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, Category="Controller")
	bool bLaunchControllerOnHost = true;
	
	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, Category="Controller", meta=(EditCondition="bLaunchControllerOnHost==true"))
	bool bUseCustomController = false;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, Category="Controller", meta=(EditCondition="bLaunchControllerOnHost==false"))
	bool bShowLaunchMessageOnHost = false;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, Category="Controller", meta=(EditCondition="bUseCustomController==true&&bLaunchControllerOnHost==true", ToolTip="Location of the Custom Controller Archive"))
	FFilePath ControllerArchivePath;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, Category="Advanced")
	bool bExportHeadless = false;

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, Category="Advanced")
	FString AlternateStartActivity = TEXT("");

	UPROPERTY(Config, EditDefaultsOnly, BlueprintReadOnly, Category="Advanced")
	bool bSkipPermissionChecks = false;
	
    USpacesRuntimeSettings(const FObjectInitializer& ObjectInitializer);

#if WITH_EDITOR

    virtual void PostInitProperties() override;
    virtual void PostEditChangeProperty(FPropertyChangedEvent& PropertyChangedEvent) override;


    /** UDeveloperSettings implementation */
    virtual FName GetContainerName() const override;
    virtual FName GetCategoryName() const override;
    virtual FName GetSectionName() const override;
    virtual FText GetSectionText() const override;
    virtual FText GetSectionDescription() const override;

	FOnUpdateAndroidConfig OnUpdateAndroidConfig;
private:
	void SetUpAndroidConfig() const;

#endif
};